# vim: set fileencoding=utf-8 :

from django.db import models
from django.db.models import Sum, Count
from django.utils.translation import ugettext_lazy as _

from datetime import datetime, date, timedelta
from dateutil.relativedelta import relativedelta
import calendar

from katrin.models import *
from katrin import service

class ServiceTraffTariff(service.ServiceTariff):
	prepaid_mb_in_month = models.DecimalField(u'Предоплаченный входящий трафик за месяц', max_digits=10, decimal_places=3, help_text=u'в мегабайтах')
	prepaid_mb_out_month = models.DecimalField(u'Предоплаченный исходящий трафик за месяц', max_digits=10, decimal_places=3, help_text=u'в мегабайтах')
	class Meta:
		db_table = u'service_traff_tariffs'
		verbose_name = u'Параметры тарифов'
		verbose_name_plural = u'Параметры тарифов'

class ServiceTraffFilter(service.ServiceFilter):
	network = models.CharField(max_length=20, default='0.0.0.0', verbose_name=_('Network'), help_text=u"%s: 192.168.0.0" % _('For example'))
	netmask = models.CharField(max_length=20, default='0.0.0.0', verbose_name=_('Netmask'), help_text=u"%s: 255.255.255.0" % _('For example'))
	port = models.IntegerField(default=-1, verbose_name=_('Port'), help_text=_("-1 if not used"))
	proto = models.IntegerField(default=-1, verbose_name=_('Protocol'), help_text=_("-1 if not used"))
	in_traff = models.BooleanField(_('Incoming traffic'), default=True)
	mb_cost = models.DecimalField(default=0, max_digits=5, decimal_places=2, verbose_name=_('1 MB cost'))
	speed_in = models.CharField(max_length=30, null=True, blank=True)
	speed_in_max = models.CharField(max_length=30, null=True, blank=True)
	speed_out = models.CharField(max_length=30, null=True, blank=True)
	speed_out_max = models.CharField(max_length=30, null=True, blank=True)
	day_hours = models.CharField(max_length=61, null=True, blank=True, verbose_name=_('Hours'), help_text=u"%s: 0,1,2,8,9,23" % _('For example'))
	week_days = models.CharField(max_length=13, null=True, blank=True, verbose_name=_('Week days'), help_text=u"%s: 1,2,3,7" % _('For example'))
	month_days = models.CharField(max_length=83, null=True, blank=True, verbose_name=_('Month days'), help_text=u"%s: 1,2,3,30,31" % _('For example'))
	year_months = models.CharField(max_length=26, null=True, blank=True, verbose_name=_('Months'), help_text=u"%s: 1,2,3,11,12" % _('For example'))
	class Meta:
		db_table = u'service_traff_filters'
		verbose_name = _('traffic filter')
		verbose_name_plural = _('Traffic filters')

Traffic = {
		'input':	1,
		'output':	0,
		}
TrafficDirection = {
		0:	_('Outgoing'),
		1:	_('Incoming'),
		}

class ServiceTraffStatLevel(service.ServiceStat):
	"""Implements common behavior for traff stat with different aggregation level"""
	filter = models.ForeignKey(ServiceTraffFilter, verbose_name=_('Filter'))
	bytes = models.PositiveIntegerField()
	in_traff = models.BooleanField()

	class Meta:
		abstract=True
	@classmethod
	def aggregate(cls, entry_time):
		start_time = entry_time
		end_time = cls.apply_stat_shift(entry_time, 1)
		queryset = cls.aggregation_source().objects.filter(date_time__range=(start_time,end_time)).\
				values('filter','client','in_traff',).annotate(cost_sum=Sum('cost'), bytes_sum=Sum('bytes'))
		for item in queryset:
			cls(
					date_time=entry_time,
					filter_id=item['filter'],
					client_id=item['client'],
					cost=item['cost_sum'],
					bytes=item['bytes_sum'],
					in_traff=item['in_traff'],
					).save()
	@classmethod
	def get_aggregated_stat(cls, start_time, end_time, group_by=('date_time','in_traff',), **also_filter_by):
		'''returns traff stat filtered by given filters and aggregated by given fields '''
		filter_by={'date_time__range':(start_time,end_time)}
		filter_by.update(also_filter_by)
		return cls.objects.filter(**filter_by).values(*group_by \
				).annotate(cost_sum=Sum('cost'), bytes_sum=Sum('bytes'))

	@classmethod
	def get_stat_summary(cls, start_time, end_time, **also_filter_by):
		stat = cls.get_aggregated_stat(start_time, end_time, ('in_traff',), **also_filter_by)
		summary = {}
		summary['download'] = 0
		summary['download_cost'] = 0
		summary['upload'] = 0
		summary['upload_cost'] = 0
		if len(stat) == 2:
			if stat[0]['in_traff'] == Traffic['input']:
				input_stat = stat[0]
				output_stat = stat[1]
			else:
				input_stat = stat[1]
				output_stat = stat[0]
			summary['download'] = input_stat['bytes_sum']
			summary['download_cost'] = input_stat['cost_sum']
			summary['upload'] = output_stat['bytes_sum']
			summary['upload_cost'] = output_stat['cost_sum']
		elif len(stat) == 1:
			if stat[0]['in_traff'] == Traffic['input']:
				summary['download'] = stat[0]['bytes_sum']
				summary['download_cost'] = stat[0]['cost_sum']
			else:
				summary['upload'] = stat[0]['bytes_sum']
				summary['upload_cost'] = stat[0]['cost_sum']
		return summary

class ServiceTraffStatMonth(ServiceTraffStatLevel):
	class Meta:
		db_table = u'service_traff_stats_month'
	@classmethod
	def apply_stat_shift(cls, date_time, sign=-1):
		# Get  previous or next month (sign=1) time
		return date_time + relativedelta(months=sign)

	@classmethod
	def aggregation_source(cls):
		return ServiceTraffStatDay


class ServiceTraffStatDay(ServiceTraffStatLevel):
	class Meta:
		db_table = u'service_traff_stats_day'
	@classmethod
	def apply_stat_shift(cls, date_time, sign=-1):
		# Get yesterday or tomorrow (sign=1) time
		return date_time + timedelta(days=sign)

	@classmethod
	def aggregation_source(cls):
		return ServiceTraffStatHour

class ServiceTraffStatHour(ServiceTraffStatLevel):
	class Meta:
		db_table = u'service_traff_stats_hour'
	@classmethod
	def apply_stat_shift(cls, date_time, sign=-1):
		# Get previous or next (sign=1) hour time
		return date_time + timedelta(hours=sign)

	@classmethod
	def aggregation_source(cls):
		return ServiceTraffStat

class ServiceTraffStat(service.ServiceStat):
	filter = models.ForeignKey(ServiceTraffFilter, verbose_name=_('Filter'))
	addr = models.PositiveIntegerField(null=True, blank=True)
	src_port = models.PositiveIntegerField(null=True, blank=True)
	dst_port = models.PositiveIntegerField(null=True, blank=True)
	bytes = models.PositiveIntegerField()
	proto = models.PositiveIntegerField(null=True, blank=True)
	in_traff = models.BooleanField()
	class Meta:
		db_table = u'service_traff_stats'
		verbose_name_plural = _('Traffic stat')

	@classmethod
	def create_usage_operations(cls, userservice, include_last_stat=False):
		klass = cls.get_stat_class(userservice)
		klass.create_usage_operations(userservice, include_last_stat)

	@classmethod
	def get_stat_class(cls, user_service):
		if user_service.tariff.billing_period.divisible==PeriodDivisibles['Hour']:
			return ServiceTraffStatHour
		elif user_service.tariff.billing_period.divisible==PeriodDivisibles['Day']:
			return ServiceTraffStatDay
		elif user_service.tariff.billing_period.divisible==PeriodDivisibles['Month']:
			return ServiceTraffStatMonth
		else:
			raise ValueError("Unknown period")

	@classmethod
	def get_stat(cls, start_time, end_time, group_by=('client',), **also_filter_by):
		'''returns detailed traff stat filtered by given filters and aggregated by given fields '''
		filter_by={'date_time__range':(start_time,end_time)}
		filter_by.update(also_filter_by)
		return cls.objects.filter(**filter_by).values(*group_by \
				).annotate(cost_sum=Sum('cost'), bytes_sum=Sum('bytes'))




ACCESS_TYPES = (
	('ip', _('Gateway')),
	('ppp', u'PPP'),
)
class ServiceTraffUparam(service.ServiceUparam):
	access_type = models.CharField(_('Access type'), max_length=45, default='ip', choices=ACCESS_TYPES)
	ip = models.CharField(_('Client IP address'), max_length=60)
	class Meta:
		db_table = u'service_traff_uparams'
		verbose_name = u"%s" % _('traffic parameters of client')
		verbose_name_plural = u"%s" % _("Traffic parameters of clients")
	def __unicode__(self):
		return u"%s" % self.client
